# Copyright (C) 2007 by Sergio Pistone
# sergio_pistone@yahoo.com.ar
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# Free Software Foundation, Inc.,
# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require "utils/strings"
require "utils/htmlentities"
require "utils/http"
require "lyrics"

class NotPopular < Lyrics

	def NotPopular.site_host()
		return "www.notpopular.com"
	end

	def NotPopular.site_name()
		return "Not Popular.com"
	end

	def NotPopular.lyrics_test_data()
		return [
			Request.new( "Radiohead", "Optimistic", "Kid A" ),
			Request.new( "Nirvana", "About a Girl", "Bleach" ),
			Request.new( "System Of A Down", "Chop Suey!", "Toxicity" ),
			Request.new( "A Perfect Circle", "The Noose", "Thirteen Step" ),
		]
	end

	def NotPopular.build_song_add_url( request )
		return "http://#{site_host()}/lyrics/addLyrics.php"
	end

	def build_lyrics_fetch_data( request )
		artist = Strings.google_search_quote( request.artist )
		album  = Strings.google_search_quote( request.album.to_s() )
		return FetchPageData.new( Strings.build_google_feeling_lucky_url( "#{artist} #{album}", "#{site_host()}/lyrics" ) )
	end

	def check_lyrics_page( request, page_body )
		md = /<title>([^<]+) - [^<]+ Lyrics - notPopular\.com<\/title>/.match( page_body )
		return md ? Strings.normalize( request.artist ) == Strings.normalize( md[1] ) : false
	end

	def parse_lyrics( response, page_body )

		page_body.tr_s!( " \n\r\t", " " )

		artist = Strings.downcase( response.request.artist.squeeze( " " ).strip() )
		if ( md = /<title>(.*) Lyrics - notPopular.com<\/title>/.match( page_body ) )
			page_title = Strings.downcase( md[1].squeeze( " " ).strip() )
			if ( idx = page_title.index( artist ) )
				response.artist = Strings.titlecase( page_title.slice( 0..idx+artist.size-1 ).strip(), true, true )
				response.album = Strings.titlecase( page_title.slice( idx+artist.size+3..-1 ).strip(), true, true )
			end
		end

		if ( md = /<\/h3>Label: [^<]+ <br>Release Year: ([0-9]+) <br>/.match( page_body ) )
			response.year = md[1]
		end

		return if ( ! page_body.gsub!( /^.*<a name="top"><\/a>/i, "" ) )
		return if ( ! page_body.gsub!( /^.*<p><a name="1"><\/a><b>/i, "" ) )
		return if ( ! page_body.gsub!( /<\/div>.*$/i, "" ) )

		md = /.*#([0-9]+)$/.match( response.url )
		track = md ? md[1] : nil
		title = Strings.normalize( response.request.title )
		page_body.split( /<br><span style="font-size: .8em;"><a href="#top">::top::<\/a><\/span><\/p><p><a name="[0-9]+"><\/a><b>/i ).each() do |song_content|
			md = /^([0-9]+)\) *([^<]+)<\/b><br ?\/?>(.*)/.match( song_content )
			next if ( ! md || ( track && track != md[1] ) || ( ! track && title != Strings.normalize( md[2] ) ) )
			response.title = md[2]
			song_content = md[3]
			song_content.gsub!( /<span style="font-size: .8em;".*$/, "" )
			song_content.gsub!( /\ ?<br ?\/?> ?/i, "\n" )
			response.lyrics = song_content
		end

	end

	def suggestions( request )

		artist_search_url = "http://#{site_host()}/lyrics/searchResults.php?SearchField=#{CGI::escape( request.artist )}"

		if ( log?() )
			log( site_name().upcase() )
			log( "Retrieving SUGGESTIONS..." )
			log( request.to_s() )
			log( " - url: #{artist_search_url}" )
			log( "Fetching artist search page... ", 0 )
		end

		response, artist_search_url = HTTP.fetch_page_get( artist_search_url )
		error = ! response || ! response.body()

		if ( log?() )
			if ( error )
				log( "ERROR" )
			else
				log( "OK" )
				log( response.body(), 2 ) if ( verbose_log?() )
			end
		end

		suggestions = []

		return suggestions if ( error )

		page_body = response.body()
		page_body.tr_s!( " \n\r\t", " " )

		return suggestions if ( ! page_body.gsub!( /^.*<h3>Artists:<\/h3>/i, "" ) )
		return suggestions if ( ! page_body.gsub!( /<h3>Albums:<\/h3>.*$/i, "" ) )

		log( "Parsing artist album pages...", 0 ) if ( log?() )

		# page_body should contain the artist's album links, we have to iterate through the album pages excerpting the lyrics links

		albums_count = 0
		normalized_album = Strings.normalize( request.album.to_s() )
		page_body.split( /<\/a><br>/ ).each() do |album_entry|

			md = /<a href="([0-9]+\/[^\/]+\/[^"]+)">/.match( album_entry )
			next if ( md == nil )

			album_url = "http://#{site_host()}/lyrics/#{md[1]}"
			albums_count += 1

			if ( log?() )
				log( "" ) if ( albums_count == 1 )
				log( " - #{album_url}" )
			end

			response, album_url = HTTP.fetch_page_get( album_url )
			next if ( ! response || ! response.body() )

			page_body = response.body()
			page_body.tr_s!( " \n\r\t", " " )

			if ( md = /<title>(.*) Lyrics - notPopular.com<\/title>/.match( page_body ) )
				insertion_idx = Strings.normalize( md[1] ).index( normalized_album ) ? 0 : -1
			else
				insertion_idx = -1
			end

			next if ( ! page_body.gsub!( /^.*<a name="top"><\/a>/i, "" ) )
			next if ( ! page_body.gsub!( /^.*<hr>/i, "" ) )
			next if ( ! page_body.gsub!( /<p><a name="1">.*$/i, "" ) )

			HTMLEntities.decode!( page_body )

			page_body.split( /<a href="#/ ).each() do |song_entry|
				if ( md = /([0-9]+)">[0-9]+\) *([^<]+)<\/a>/.match( song_entry ) )
					suggestion = Suggestion.new( request.artist, md[2].strip(), "#{album_url}##{md[1]}" )
					suggestions.insert( insertion_idx, suggestion )
					insertion_idx += 1 if ( insertion_idx >= 0 )
					log( "    - #{suggestion}" ) if ( log?() )
				end
			end

		end

		if ( albums_count == 0 && log?() )
			log( " NO ALBUM PAGES AND SUGGESTIONS FOUND" )
		end

		return suggestions

	end

end
